@@ -1476,6 +1476,8 @@ UPDATE alembic_version SET version_num='37a5332640e2' WHERE alembic_version.vers
 
 -- Running upgrade 37a5332640e2 -> 24c12d8e9014
 
+ALTER TABLE ps_registrations ADD COLUMN user_agent VARCHAR(255);
+
 UPDATE alembic_version SET version_num='24c12d8e9014' WHERE alembic_version.version_num = '37a5332640e2';
 
 -- Running upgrade 24c12d8e9014 -> bd335bae5d33
@@ -1641,8 +1643,6 @@ UPDATE alembic_version SET version_num='74dc751dfe8e' WHERE alembic_version.vers
 
 ALTER TABLE ps_transports ADD COLUMN tcp_keepalive_enable BOOL;
 
-ALTER TABLE ps_transports ADD CHECK (tcp_keepalive_enable IN (0, 1));
-
 ALTER TABLE ps_transports ADD COLUMN tcp_keepalive_idle_time INTEGER;
 
 ALTER TABLE ps_transports ADD COLUMN tcp_keepalive_interval_time INTEGER;
@@ -1689,3 +1689,27 @@ ALTER TABLE ps_endpoint_id_ips DROP COLUMN transport;
 
 UPDATE alembic_version SET version_num='bd9c5159c7ea' WHERE alembic_version.version_num = '6c475a93f48a';
 
+-- Running upgrade bd9c5159c7ea -> 2b7c507d7d12
+
+ALTER TABLE queues ADD COLUMN log_restricted_caller_id ENUM('0','1','off','on','false','true','no','yes');
+
+UPDATE alembic_version SET version_num='2b7c507d7d12' WHERE alembic_version.version_num = 'bd9c5159c7ea';
+
+-- Running upgrade 2b7c507d7d12 -> 655054a68ad5
+
+ALTER TABLE ps_endpoints ADD COLUMN tenantid VARCHAR(80);
+
+UPDATE alembic_version SET version_num='655054a68ad5' WHERE alembic_version.version_num = '2b7c507d7d12';
+
+-- Running upgrade 655054a68ad5 -> 801b9fced8b7
+
+ALTER TABLE ps_subscription_persistence ADD COLUMN generator_data TEXT;
+
+UPDATE alembic_version SET version_num='801b9fced8b7' WHERE alembic_version.version_num = '655054a68ad5';
+
+-- Running upgrade 801b9fced8b7 -> 4f91fc18c979
+
+ALTER TABLE ps_endpoints ADD COLUMN suppress_moh_on_sendonly ENUM('0','1','off','on','false','true','no','yes');
+
+UPDATE alembic_version SET version_num='4f91fc18c979' WHERE alembic_version.version_num = '801b9fced8b7';
+
