@@ -130,8 +130,6 @@
 							<enum name="ast_channel_hangup_handler_type" />
 							<enum name="ast_channel_moh_start_type" />
 							<enum name="ast_channel_moh_stop_type" />
-							<enum name="ast_channel_monitor_start_type" />
-							<enum name="ast_channel_monitor_stop_type" />
 							<enum name="ast_channel_mixmonitor_start_type" />
 							<enum name="ast_channel_mixmonitor_stop_type" />
 							<enum name="ast_channel_mixmonitor_mute_type" />
@@ -139,6 +137,7 @@
 							<enum name="ast_channel_agent_logoff_type" />
 							<enum name="ast_channel_talking_start" />
 							<enum name="ast_channel_talking_stop" />
+							<enum name="ast_channel_tone_detect" />
 							<enum name="ast_security_event_type" />
 							<enum name="ast_named_acl_change_type" />
 							<enum name="ast_local_bridge_type" />
